// GrassCl include file
// Macros for generating grass with seeds.
//
// Brendt W. Hess, 1999
// Released to the public domain - modify to your heart's content.
//
//  enjoy!
//
 
// To create clumps with no middle stem, define NoStem before calling MakeClump, e.g.
//  #declare NoStem=1; 
//  This has not been tested to completion - use at your own risk
//

// To use the MakeClump macro:
//      MakeClump(ColorIndex)
//
//      where ColorIndex is between 0 and 5
//      Colorindex references an array of pigments (GrassPigments[])
//          If you want to use your own pigments, set them up below.
//

#include "colors.inc"
#include "textures.inc"

#ifndef (Sd1)       // do this once when first used
    #declare Sd1=seed(2000);
#end

/*
//  pigments for dry grasses

#declare GrassPigments=array[6] {pigment{rgb <1, 0.9, 0.58>},
                                 pigment{rgb <.98,.94, .592>},
                                 pigment{rgb <.96,.94, .56>},
                                 pigment{rgb <.99, .89, .4>},
                                 pigment{rgb <.8, .915, 0.318>},
                                 pigment{rgb <1, 0.95, 0.610>} }
*/

//  pigments for decorative grasses

#declare GrassPigments=array[6] {pigment{rgb <.2, 0.5, 0.38>},
                                 pigment{rgb <.27,.74, .05>},
                                 pigment{rgb <.0,.55, .12>},
                                 pigment{marble turbulence 0.33
                                            color_map {
                                                        [0 rgb <.8, .9, .8>]
                                                        [0.06 rgb <.8, .9, .8>]
                                                        [0.12 rgb <.7,.3,.1>]
                                                        [0.20 rgb <.5,.5,.11>]
                                                        [0.5  rgb <.12,.53,.22>]
                                                        [0.80 rgb <.5,.5,.11>]
                                                        [0.88 rgb <.7,.3,.1>]
                                                        [0.94 rgb <.8, .9, .8>]
                                                        [1 rgb <.8, .9, .8>]
                                                        }
                                                    },
                                 pigment{rgb <.376, .525, 0.118>},
                                 pigment{rgb <.6, 0.8, 0.4>} }

  
#declare SeedTex=texture { 
        pigment { 
            bozo
            color_map {
              [0.0 rgb <.95,.945,.689>]  
              [0.8 rgb <.94, 0.92, 0.67>]
              [0.9 rgb <.92, 0.91, 0.62>]
              [1.0 rgb <.97, 0.93, 0.50>]
            }
        turbulence 0.95
        scale <0.5, 0.3, 0.4>
      }
      finish { ambient .2 diffuse .6 }
}

#macro MakeSeed(Ct)
    #local SeedCount=Ct;
    #local SeedLoop=SeedCount;     
    #local SSize=SeedLoop*1.2;
    #local SeedCutoff=int(sqrt(SeedLoop));
    object {
    intersection {
        sphere {<-SeedLoop,0,0> SSize }
        sphere {<SeedLoop,0,0> SSize }
        #while (SeedLoop>SeedCutoff)
            sphere {<sqrt(pow(SeedCount,2)-pow(SeedLoop,2)),0,SeedLoop> SSize }
            sphere {<-sqrt(pow(SeedCount,2)-pow(SeedLoop,2)),0,SeedLoop> SSize }
            sphere {<sqrt(pow(SeedCount,2)-pow(SeedLoop,2)),0,-SeedLoop> SSize }
            sphere {<-sqrt(pow(SeedCount,2)-pow(SeedLoop,2)),0,-SeedLoop> SSize }
            #local SeedLoop=SeedLoop-1;
        #end
        texture {SeedTex translate <rand(Sd1),rand(Sd1),rand(Sd1)>}
        scale <-.5,1,-.5>
    } bounded_by {sphere {<0,0,0> SSize*.66}}}
#end

#declare SeedBody=MakeSeed(6) 

#declare SeedBody2=MakeSeed(6)

#macro SeedWhisker(Segs)    
#local L=0;
#local W=.01;
#local YP=4;    
#local Base=.16;
#local TiltRate=.02+.016*rand(Sd1);   
#local Tilt=0;
#local XOfs=0;                                
object {
merge {
    #while (L<Segs)
        #declare BW=Base-W;
        cone {<XOfs,YP,0>, Base
              <XOfs+Tilt,YP+1.01,0>, BW
              texture { pigment { GrassPigments[1]} finish { ambient .2 diffuse .6 }}
              }
        #local Base=Base-W;
        #local L=L+1;
        #local YP=YP+1;
        #local XOfs=XOfs+Tilt;
        #local Tilt=Tilt+TiltRate;
    #end      
    }   
    bounded_by{ box {<-.1602,-.02,-.1602>, <XOfs+Base+W,YP+.011,.1602>}}
    }
#end     
                                                       
                                                       
#declare FlatBlade=object { union {

  prism {
    cubic_spline
    0, // sweep the following shape from here ...
    8, // ... up through here
    11, // the number of points making up the shape ...
    < -0.08, -0.2>, // point#1 (control point... not on curve)
    < 0,  0>, // point#2  ... THIS POINT ...
    <-0.08,  0.2>, // point#3
    <-0.15, 1>,
    <0,0.2>,
    <0.1,0>,
    <0,-0.2>,
    <-0.15,-1>,    
    < -0.08, -0.2>, // point#4
    < 0,  0>, // point#5 ... MUST MATCH THIS POINT
    <-0.08,  0.2>  // point#6 (control point... not on curve)
    rotate x*180
  }
  
  prism {       
    conic_sweep
    cubic_spline
    0.025, // sweep the following shape from here ...
    0.5, // ... up through here
    11, // the number of points making up the shape ...
    < -0.08, -0.2>, // point#1 (control point... not on curve)
    < 0,  0>, // point#2  ... THIS POINT ...
    <-0.08,  0.2>, // point#3
    <-0.15, 1>,
    <0,0.2>,
    <0.1,0>,
    <0,-0.2>,
    <-0.15,-1>,    
    < -0.08, -0.2>, // point#4
    < 0,  0>, // point#5 ... MUST MATCH THIS POINT
    <-0.08,  0.2>  // point#6 (control point... not on curve)
    rotate x*180
    translate <0,2.5,0>
    scale <1,2,1>
  }
  
  prism {       
    conic_sweep
    cubic_spline
    0.5, // sweep the following shape from here ...
    1, // ... up through here
    11, // the number of points making up the shape ...
    < -0.08, -0.2>, // point#1 (control point... not on curve)
    < 0,  0>, // point#2  ... THIS POINT ...
    <-0.08,  0.2>, // point#3
    <-0.15, 1>,
    <0,0.2>,
    <0.1,0>,
    <0,-0.2>,
    <-0.15,-1>,    
    < -0.08, -0.2>, // point#4
    < 0,  0>, // point#5 ... MUST MATCH THIS POINT
    <-0.08,  0.2>  // point#6 (control point... not on curve)
    rotate x*180
    translate <0,1,0>
    scale <1,8,1>
  }             
  rotate y*180
  scale <1,1,0.6>
}
}


#macro GrassStem(Segs, ColorIndex)    
#local L=0;                         // loop
#local W=.003;                     // Cone Width Adjustment
#local YP=0;                        // Start of each cone
#local Base=.008*Segs;                     // starting cone width
#local TiltRateX=.005*rand(Sd1)+.01;                           // calculate tilt rate
#local TiltRateZ=.005*rand(Sd1)+.01;
#local TiltRate=sqrt(pow(TiltRateX,2)+pow(TiltRateZ,2));
#local TiltX=.01*rand(Sd1)+.005;
#local XOfs=0;                      // start a x=0
#local YLen=0.8+0.4*rand(Sd1);     // stem segment length
#local Idx=int(abs(ColorIndex));
#if (ColorIndex>5)
    #local Idx=0;
#end
#local StemColor=GrassPigments[Idx]

union {
    #while (L<Segs-2)
        #declare BW=Base-W;                         // narrow end of cone
        cone {<XOfs,YP,0>, Base
              <XOfs+TiltX,YP+YLen+.02,0>, BW
              texture { pigment {StemColor} finish { ambient .2 diffuse .6 }}
              }
        #local Base=BW;                             // narrow working base by step
        #local L=L+1;                               // count
        #local YP=YP+YLen;                          // adjust Y position by Y step
        #local XOfs=XOfs+TiltX;                     // adjust X position by X step
        #local TiltX=TiltX+TiltRate;                // adjust X tilt step
    #end

    #local XYDist=sqrt(pow(YLen,2)+pow(XOfs-TiltX,2));
    #local XYDist2=sqrt(pow(YLen,2)+pow(XOfs,2));
    #local XAngle=degrees(asin(pow(XOfs-TiltX,2)/pow(XYDist,2)));
    #local XAngle2=degrees(asin(pow(XOfs,2)/pow(XYDist2,2))); 
    #local Rate=XAngle2-XAngle;
    #local L=0;   
    #local TiltX=XAngle2;
    #while (L<4)
        #local W=W*.97;
        #local YL=YLen*(0.9-0.15*L);
        #if (TiltX>40)
            #local Rate=Rate*1.03;
        #else
            #if (TiltX<20)
                #local Rate=Rate*1.25;
            #else
                #local Rate=Rate*1.1;
            #end
        #end
        #local TiltX=TiltX+Rate;
        #local ZX=sin(radians(TiltX))*YL;
        #local ZY=sin(radians(90-TiltX))*YL;
        #local BW=Base-W;
        cone {<XOfs,YP,0>, Base
              <XOfs+ZX,YP+ZY+.01,0>, BW
              texture { pigment {StemColor} finish { ambient .2 diffuse 0.6 }}
              }
        #local Base=BW;                             // narrow working base by step
        #local L=L+1;                               // count
        #local YP=YP+ZY;                            // adjust Y position by Y step

        #local XOfs=XOfs+ZX;                     // adjust X position by X step
    #end


    #local W=W*.8;                                // smaller steps on narrowing
    #local L=0;
    #local YLen=YLen/6;                           // seedhead segments smaller
    #while (L<11)                                   // 10 steps
        #local W=W*.97;
        #local Rate=Rate*1.1;
        #local TiltX=TiltX+Rate;
        #local ZX=sin(radians(TiltX))*YLen;
        #local ZY=sin(radians(90-TiltX))*YLen;
        #local BW=Base-W;                         // narrow end of cone
        cone {<XOfs,YP,0>, Base
              <XOfs+ZX,YP+ZY+.01,0>, BW
              texture { pigment {StemColor} finish { ambient .2 diffuse 0.6}}
              }
        #if (L>3)
            #local SW1=object{SeedWhisker(9+int(4*rand(Sd1))) translate <0,-.5,0>}
            #local SW2=object{SeedWhisker(9+int(4*rand(Sd1))) translate <0,-.5,0>}
            #local Se1=object { 
                            union { 
                                object {SeedBody scale (1-L/50)}
                                SW1
                                }
                              }
            #local Se2=object { 
                            union { 
                                object {SeedBody2 scale (1-L/50)}
                                SW2
                                }
                              }
            object {Se1 scale .046
                        rotate -(TiltX+rand(Sd1)*5-2.5)*z
                        rotate -(40-L+rand(Sd1)*.9-.45)*y 
                        translate <XOfs,YP+ZY/2.5,Base+.048+L/800>
                   }
    
            object {Se2 scale .046
                        rotate -(TiltX+rand(Sd1)*5-2.5)*z 
                        rotate (40-L+rand(Sd1)*.9-.45)*y
                        translate <XOfs+ZX,YP+ZY,-(BW+.048-L/800)>
                        
                   }
        #end                        
        #local Base=Base-W;
        #local L=L+1;
        #local YP=YP+ZY;
        #local XOfs=XOfs+ZX;
    #end
    }
#end     

#declare object_axis1=<0,0,0>;
#declare object_axis2=<0,13,0>;

#macro Blade(ScaleFactor, ColorIndex)            
    #local Idx=ColorIndex;
    #if (Idx>5)
        #local Idx=0;
    #end
    #local BladeColor=GrassPigments[Idx]
    #declare bend_object=object{FlatBlade translate <0,8,0> pigment {BladeColor}}
    object {
        #declare bend_angle=40*rand(Sd1)+20;
        #declare bend_direction=x;
        #declare bend_start=0.05*rand(Sd1)+.01;
        #declare bend_finish=0.08*rand(Sd1)+0.92;
        #include "bend.inc"
        scale ScaleFactor
        }
#end          

#macro MakeClump(ColorCode)
        #declare ZLoop=0;
        #declare ZCount=6+int(rand(Sd1)*5);
        #if (ColorCode<0)
            #declare ClumpColor=int(rand(Sd1)*6);
        #else
            #declare ClumpColor=ColorCode;
        #end
        object {
            union {
            #while (ZLoop<ZCount)
                #declare XX=rand(Sd1);
                #if (XX>.3)
                    #declare BL=object{Blade(0.24+rand(Sd1)*.12, ClumpColor) scale <1,0.8+rand(Sd1)*.4,1>}
                    #if (XX>.85)    
                        #declare BL2=object{Blade(0.24+rand(Sd1)*.12, ClumpColor) scale <1,0.8+rand(Sd1)*.4,1> rotate y*(rand(Sd1)*10+2)}
                        #declare BL1=object{Blade(0.24+rand(Sd1)*.12, ClumpColor) scale <1,0.8+rand(Sd1)*.4,1> rotate -y*(rand(Sd1)*10+2)}
                        #declare BL=union {object{BL1}
                                           object{BL2}
                                            }
                                          
                    #end
                #else
                    #declare BL=object{sphere{<-0.18,0,0>, 0.01 pigment{color Clear}}}
                #end                                          
                
                #declare Rot=ZLoop*360/ZCount+rand(Sd1)*20-10;
                #ifdef (NoStem)
                    object {BL translate <0.18,0,0> 
                        rotate -z*8*rand(Sd1) rotate y*(rand(Sd1)*18-9) translate x*(rand(Sd1)*.4+.03) rotate y*Rot 
                        }
                    
                    
                #else                    
                    union{
                        GrassStem(7,ClumpColor) 
                        object {BL translate <0.18,0,0>}
                        
                        rotate -z*8*rand(Sd1) rotate y*(rand(Sd1)*18-9) translate x*(rand(Sd1)*.4+.03) rotate y*Rot 
                      }
                #end
                #declare ZLoop=ZLoop+1;
            #end
            }       
       }
#end

